import React, {Component} from 'react'
import PropTypes from 'prop-types'
import {css} from 'glamor'
import Box from './Box'
import Overlay from './Overlay'
import CloseIcon from './Close'
import theme from './theme'
import offscreenCss from './utils/offscreenCss'
const {majorScale, minorScale, borderRadius, colors, rawColors, textStyles} = theme

const overlayStyles = {
  display: 'flex',
  alignItems: 'center',
  justifyContent: 'center'
}

const mainCss = css({
  position: 'relative',
  maxWidth: majorScale(72),
  border: `1px solid ${colors.borderColor}`,
  borderRadius: borderRadius[1],
  background: colors.white,
  animationDuration: theme.animationDuration.fast,
  animationTimingFunction: theme.animationEasing.standard,
  animationFillMode: 'forwards'
})

const titleCss = css({
  display: 'flex',
  alignItems: 'center',
  justifyContent: 'space-between',
  padding: `${majorScale(2)} ${majorScale(3)}`,
  borderBottom: `1px solid ${colors.borderColor}`,
  ...textStyles.display3
})

const buttonCss = css({
  padding: majorScale(1),
  marginRight: `-${majorScale(1)}`,
  background: 'none',
  cursor: 'pointer',
  color: rawColors.grey300,
  lineHeight: 0,
  ':hover, :focus': {
    outline: 'none',
    color: rawColors.grey400
  }
})

const openAnimation = css.keyframes('openAnimation', {
  from: {
    transform: 'scale(0.8)',
    opacity: 0
  },
  to: {
    transform: 'scale(1)',
    opacity: 1
  }
})

const closeAnimation = css.keyframes('closeAnimation', {
  from: {
    transform: 'scale(1)',
    opacity: 1
  },
  to: {
    transform: 'scale(0.8)',
    opacity: 0
  }
})

const parseProps = ({
  className,
  children,
  title,
  isOpen,
  onClose,
  ...props
}) => props

class Dialog extends Component {
  constructor (props) {
    super()

    this.state = {
      isMounted: props.isOpen,
      isClosing: false
    }

    this.handleFadeEnd = this.handleFadeEnd.bind(this)
  }

  componentWillReceiveProps (nextProps) {
    if (!this.props.isOpen && nextProps.isOpen) {
      this.setState({isMounted: true, isClosing: false})
    }

    if (this.props.isOpen && !nextProps.isOpen) {
      this.setState({isMounted: true, isClosing: true})
    }
  }

  render () {
    const {title, onClose, children} = this.props
    const {isMounted, isClosing} = this.state

    if (!isMounted) return null

    const style = {
      animationName: `${isClosing ? closeAnimation : openAnimation}`
    }

    return (
      <Overlay
        onClick={onClose}
        fadeOut={isClosing}
        onFadeEnd={this.handleFadeEnd}
        {...overlayStyles}
      >
        <Box
          {...parseProps(this.props)}
          is='section'
          className={`${mainCss}`}
          style={style}
          role='dialog'
          aria-label={title}
        >
          <Box is='h2' className={`${titleCss}`}>
            {title}
            {onClose && (
              <Box is='button' className={`${buttonCss}`} onClick={onClose}>
                <div className={`${offscreenCss}`}>Close</div>
                <CloseIcon width={minorScale(3)} height={minorScale(3)} />
              </Box>
            )}
          </Box>
          <Box padding={majorScale(3)}>
            {children}
          </Box>
        </Box>
      </Overlay>
    )
  }

  handleFadeEnd () {
    if (!this.state.isClosing) return
    this.setState({isMounted: false, isClosing: false})
  }
}

Dialog.propTypes = {
  ...Box.propTypes,
  title: PropTypes.node.isRequired,
  isOpen: PropTypes.bool.isRequired,
  children: PropTypes.node.isRequired,
  onClose: PropTypes.func
}

Dialog.defaultProps = {
  isOpen: false
}

Dialog.displayName = 'Dialog'

export default Dialog
