import React, { Component } from 'react'
import PropTypes from 'prop-types'
import { css } from 'glamor'
import Box from './Box'
import Pane from './Pane'
import Overlay from './Overlay'
import Close from './Close'
import theme from './theme'
const { majorScale } = theme

const rotate360InAnimation = css.keyframes('rotate360InAnimation', {
  from: {
    transform: `translateX(100%) rotate(0deg)`
  },
  to: {
    transform: `translateX(-100%) rotate(-360deg)`
  }
})

const rotate360OutAnimation = css.keyframes('rotate360OutAnimation', {
  from: {
    transform: `translateX(-100%) rotate(0deg)`
  },
  to: {
    transform: `translateX(100%) rotate(360deg)`
  }
})

const SheetClose = ({ isClosing, ...props }) =>
  <Box {...css({
    cursor: 'pointer',
    transform: `translateX(-100%)`,
    backgroundColor: `rgba(255, 255, 255, 0.4)`,
    transition: `background-color ${theme.animationDuration.fast}`,
    animation: `${isClosing ? rotate360OutAnimation : rotate360InAnimation} ${theme.animationDuration.slow} ${theme.animationEasing.deceleration}`,
    '&:hover': {
      backgroundColor: `rgba(255, 255, 255, 0.6)`
    },
    '&:active': {
      backgroundColor: `rgba(255, 255, 255, 0.4)`
    }
  })} {...props}>
    <Close fill='#fff' display='block' />
  </Box>

SheetClose.defaultProps = {
  position: 'absolute',
  marginLeft: majorScale(-2),
  marginTop: majorScale(2),
  padding: majorScale(1),
  circle: true
}

const slideInAnimation = css.keyframes('slideInAnimation', {
  from: {
    transform: `translateX(100%)`
  },
  to: {
    transform: `translateX(0)`
  }
})

const slideOutAnimation = css.keyframes('slideOutAnimation', {
  from: {
    transform: `translateX(0)`
  },
  to: {
    transform: `translateX(100%)`
  }
})

const SheetPane = ({ slideOut, ...props }) => <Pane {...css({
  transform: `translateZ(0)`,
  animation: `${slideOut ? slideOutAnimation : slideInAnimation} ${theme.animationDuration.slow} ${theme.animationEasing.deceleration}`
})} {...props} />

SheetPane.propTypes = {
  slideOut: PropTypes.bool
}

SheetPane.defaultProps = {
  white: true,
  position: 'absolute',
  shadow: true,
  top: 0,
  right: 0,
  bottom: 0
}

SheetPane.displayName = 'SheetPane'

class Sheet extends Component {
  constructor () {
    super()

    this.handleClose = this.handleClose.bind(this)
    this.handleFadeEnd = this.handleFadeEnd.bind(this)

    this.state = {
      isClosing: false
    }
  }

  handleClose () {
    this.setState({ isClosing: true })
  }

  handleFadeEnd () {
    if (!this.state.isClosing) return
    if (this.props.onClose) { this.props.onClose() }
    this.setState({ isClosing: false })
  }

  render () {
    const { isOpen, children } = this.props
    const { isClosing } = this.state

    if (!isOpen) return null

    return (
      <Overlay onClick={this.handleClose} fadeOut={isClosing} onFadeEnd={this.handleFadeEnd}>
        <SheetPane slideOut={isClosing} {...this.parseProps(this.props)}>
          <SheetClose isClosing={isClosing} onClick={this.handleClose} />
          {children}
        </SheetPane>
      </Overlay>
    )
  }

  parseProps ({
    className,
    children,
    isOpen,
    onClose,
    ...props
  }) { return props }
}

Sheet.propTypes = {
  ...Box.propTypes,
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func
}

Sheet.defaultProps = {
  width: majorScale(85),
  isOpen: true
}

Sheet.displayName = 'Sheet'

export default Sheet
